SystemOrganization addCategory: #'Widgets-Core'!
SystemOrganization addCategory: #'Widgets-Support'!
SystemOrganization addCategory: #'Widgets-Designer'!
SystemOrganization addCategory: #'Widgets-Views'!
SystemOrganization addCategory: #'Widgets-Views-Examples'!
SystemOrganization addCategory: #'Widgets-Views-Support'!
SystemOrganization addCategory: #'Widgets-Dialogs'!

!UiPropertyTable methodsFor: 'adding - combined' stamp: 'mt 7/16/2010 11:16:33.798'!
addAllProperties: aMorph

	| editor |
	self addSeparator: 'General'.
	
	self addPropertyLabel: 'Type' editor: (UiLabel new text: aMorph class asString).
	
	((aMorph isKindOf: UiContainer) and: [(aMorph owner isKindOf: UiContainer) not])
		ifTrue: [
			editor := self createEditorFor: String value: aMorph uiClassName.
			self addPropertyLabel: 'UI Name' editor: editor.
			editor connect: editor signal: #valueChanged: to: aMorph selector: #uiClassName:.]
		ifFalse: [
			editor := self createEditorFor: String value: aMorph objectName.
			self addPropertyLabel: #Name editor: editor.
			editor connect: editor signal: #valueChanged: to: aMorph selector: #objectName:].
		
	editor := self createEditorFor: String value: aMorph objectDescription.
	self addPropertyLabel: #Description editor: editor.
	editor connect: editor signal: #valueChanged: to: aMorph selector: #objectDescription:.
	
	self addSeparator: 'Geometry'.
	
"	self addIntegerProperty: #left morph: aMorph."
	self addPointProperty: #topLeft morph: aMorph.
	self addIntegerProperty: #width morph: aMorph.
	self addIntegerProperty: #height morph: aMorph.
	
	self addSeparator: 'General Layout'.
	
	self addListProperty: #hResizing values: #(rigid spaceFill shrinkWrap) morph: aMorph.
	self addListProperty: #vResizing values: #(rigid spaceFill shrinkWrap) morph: aMorph.
	
	editor := self createEditorFor: Boolean value: (aMorph layoutPolicy notNil and: [aMorph layoutPolicy isTableLayout]).
	self addPropertyLabel: 'Table Layout' editor: editor.
	self connect: editor signal: #toggled: toSelector: #changeTableLayout:enable: pattern: {#=. aMorph. 1}.
	
	aMorph hasTableLayout ifTrue: [self addTableLayoutProperties: aMorph].
	
	self addCustomProperties: aMorph.! !

!UiComboBox methodsFor: 'accessing' stamp: 'mt 9/20/2010 14:09:40.81'!
color: aColor

	readOnlyButton color: aColor.
	button color: aColor.! !

!UiComboBox methodsFor: 'initialization' stamp: 'mt 9/22/2010 09:41:34.297'!
initialize

	super initialize.
	
	self
		vResizing: #shrinkWrap;
		changeTableLayout;
		listDirection: #leftToRight;
		cellPositioning: #topLeft;
		width: 150;
		cellInset: 1.
		
	super color: Color transparent.
		
	button := UiPushButton new
		text: nil;
		checkable: true;
		icon: UiIcons comboBoxArrowIcon;
		layoutInset: (3@4 corner: 3@4).
	lineEdit := UiLineEdit new.
	listWidget := UiListWidget new autoDeselect: false; vResizing: #shrinkWrap.
	
	readOnlyButton := UiPushButton new
		checkable: true;
		hResizing: #spaceFill.
	
	"Initialize the submorphs the first time."
	self readOnly: true.
		
	self connect: listWidget signal: #currentRowChanged: toSelector: #updateCurrentRow:.
	self connect: listWidget signal: #currentRowChanged: toSelector: #hideList.
	self connect: button signal: #toggled: toSelector: #listVisible:.
	self connect: button signal: #pressed toSelector: #showList.
	self connect: readOnlyButton signal: #pressed toSelector: #showList.
	
	self connect: readOnlyButton signal: #toggled: to: button selector: #checked:.
	self connect: button signal: #toggled: to: readOnlyButton selector: #checked:.
! !

!UiComboBox methodsFor: 'private' stamp: 'mt 7/1/2010 18:02:33.068'!
showList

	listWidget
		updateListExtent;
		width: (self width max: listWidget width);
		topLeft: (self localPointToGlobal: self bottomLeft) - (1@1);
		openInWorld.! !

!UiComboBox methodsFor: 'updating' stamp: 'mt 9/22/2010 09:41:27.942'!
updateSubmorphs

	self removeAllMorphs.
	
	self
		addMorphBack: (self readOnly ifTrue: [readOnlyButton] ifFalse: [lineEdit]);
		addMorphBack: button.! !

!UiPushButton methodsFor: 'drawing' stamp: 'mt 9/1/2010 12:28:06.513'!
drawOn: aCanvas

	| currentColor |
	currentColor := self color.
	
	self borderStyle: (
		(self down or: [self checked])
			ifFalse: [BorderStyle raised width: 2]
			ifTrue: [BorderStyle inset width: 2]).
	
	(self down or: [self checked])
		ifTrue: [color := currentColor darker].
	
	super drawOn: aCanvas.
	
	color := currentColor.
	
	aCanvas
		frameRoundRect: self outerBounds
		radius: 3
		width: 1
		color: (Color gray: 0.3).! !

!UiDesigner methodsFor: 'save/load' stamp: 'mt 6/30/2010 12:39:44.72'!
save
	
	| generator |
	generator := UiCodeGenerator new
		category: categoryEdit theText;
		source: container.
	
	(Smalltalk includesKey: generator uiClassName asSymbol)
		ifTrue: [(UserDialogBoxMorph
			confirm:  'The UI class already exists:', String cr, '[', generator category, '] ', generator uiClassName
			title: 'Replace UI class?'
			trueChoice: 'Replace UI class.'
			falseChoice: 'Cancel.'
			at: self center) ifFalse: [^ self]].
	
	generator generateUiClassFor: container.
		
	UserDialogBoxMorph
		inform: 'UI class was created successfully to:', String cr, '[', generator category, '] ', generator uiClassName
		title: 'Done.'
		at: self center.! !

!UiToolButton methodsFor: 'drawing' stamp: 'mt 9/1/2010 12:29:58.136'!
drawOn: aCanvas

	| currentColor |
	currentColor := self color.
	
	self borderStyle: (
		(self down or: [self checked])
			ifFalse: [(self autoRaise not or: [self raised]) ifTrue: [BorderStyle raised width: 1] ifFalse: [nil]]
			ifTrue: [BorderStyle inset width: 1]).
	
	(self down or: [self checked])
		ifTrue: [color := currentColor darker].
	
	super drawOn: aCanvas.
	
	color := currentColor.	! !

!UiCodeGenerator methodsFor: 'private - inst vars' stamp: 'mt 7/21/2010 11:25:25.87'!
injectInstVarsForMorph: aMorph into: aCollection

	"Root panel already has an instance variable."
	aMorph ~~ source ifTrue: [
		aMorph
			setProperty: #objectNameTemp
			toValue: (aMorph objectName ifEmpty: [self nextDefaultName]).
		
		aMorph objectDescription ifEmpty: [aMorph objectDescription: 'No description given.'].
		aCollection add: (aMorph valueOfProperty: #objectNameTemp) -> aMorph objectDescription].
	(aMorph isKindOf: UiContainer)
		ifTrue: [
			aMorph submorphs do: [:m | self injectInstVarsForMorph: m into: aCollection]].! !

!UiCodeGenerator methodsFor: 'private - reading' stamp: 'mt 7/21/2010 11:37:55.248'!
injectUiCodeFor: aMorph into: stream

	| creationTemplate submorphTemplate nameTemplate descrTemplate |
	creationTemplate := '{1} := {2} new.'.
	submorphTemplate := '{1} addMorphBack: {2}.'.
	nameTemplate := '{1} setProperty: #objectName toValue: ''{1}''.'.
	descrTemplate := '{1} setProperty: #objectDescription toValue: ''{2}''.'.

	"Create aMorph."
	stream crtab; nextPutAll: (creationTemplate format: {
		aMorph valueOfProperty: #objectNameTemp.
		aMorph class name}).
	
	"Initialize aMorph."
	aMorph hasTableLayout
		ifTrue: [stream crtab; nextPutAll: ('{1} changeTableLayout.' format: {
			aMorph valueOfProperty: #objectNameTemp})].
	
	#(extent position hResizing vResizing cellInset cellPositioning cellSpacing layoutInset listCentering listDirection listSpacing maxCellSize minCellSize wrapCentering wrapDirection) do: [:property |
		self injectUiCodeFor: aMorph property: property into: stream].
	
	self injectCustomUiCodeFor: aMorph into: stream.
	
	stream crtab; nextPutAll: (nameTemplate format: {
		aMorph valueOfProperty: #objectNameTemp.
		aMorph objectName.}).
	stream crtab; nextPutAll: (descrTemplate format: {
		aMorph valueOfProperty: #objectNameTemp.
		aMorph objectDescription}).
	
	"Let my submorphs initialize."
	(aMorph isKindOf: UiContainer)
		ifTrue: [
			aMorph submorphs do: [:m |
				self injectUiCodeFor: m into: stream.
				stream crtab; nextPutAll: (submorphTemplate format: {
					aMorph valueOfProperty: #objectNameTemp.
					m valueOfProperty: #objectNameTemp.})]].! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 7/21/2010 12:31:10.481'!
nextDefaultName

	defaultNameCount := defaultNameCount + 1.
	^ #morphy, defaultNameCount asString asSymbol! !
